import { Context } from "../reactive/signal";
import { StateNode, SetStateFunction, State } from "../reactive/state";
import { Component } from "./component";
declare type SuspenseState = "running" | "suspended" | "fallback";
declare type SuspenseContextType = {
    increment?: () => void;
    decrement?: () => void;
    state?: () => SuspenseState;
    initializing?: boolean;
};
declare type SuspenseConfig = {
    timeoutMs: number;
};
export declare const SuspenseContext: Context<SuspenseContextType> & {
    transition?: {
        timeoutMs: number;
        increment(): void;
        decrement(): void;
    };
    active?(): boolean;
    increment?(): void;
    decrement?(): void;
};
export declare function awaitSuspense(fn: () => any): () => Promise<unknown>;
export declare function createResource<T>(value?: T): [() => T | undefined, (p?: Promise<T>) => () => boolean];
export interface LoadStateFunction<T> {
    (v: {
        [P in keyof T]?: Promise<T[P]> | T[P];
    }, reconcilerFn?: (v: Partial<T>) => (state: State<T>) => void): {
        [P in keyof T]: boolean;
    };
}
export declare function createResourceState<T extends StateNode>(state: T | State<T>): [State<T>, LoadStateFunction<T>, SetStateFunction<T>];
export declare function lazy<T extends Component<any>>(fn: () => Promise<{
    default: T;
}>): T;
export declare function useTransition(config: SuspenseConfig): [() => boolean, (fn: () => any) => void];
export declare function suspend<T>(fn: () => T): () => T;
declare type HydrationContext = {
    id: string;
    count: number;
    registry?: Map<string, Element>;
};
declare type GlobalHydration = {
    context?: HydrationContext;
};
declare global {
    var _$HYDRATION: GlobalHydration;
}
export {};
